<?php
/**
 * This little script will convert the old style PHP compat-*.inc files into XML
 * and namespace the HTML 4 elements.
 */

function html2xhtml($string) {
	/* Old HTML-tags */
	$pattern = array(
		'/<b>(.+)<\/b>/is',
		'/<u>(.+)<\/u>/is',
		'/<i>(.+)<\/i>/is',
		'/<span style="color:\s*(.+)">(.+)<\/span>/is',
		'/<a href="(.+)">(.+)<\/a>/is',
		'/<br>/is',
		'/&(?!amp;)/is',
	);
	/* New HTML-tags (namespaced for XML use). */
	$replace_ns = array(
		'<h:span class="bold">\\1</h:span>',
		'<h:span class="underline">\\1</h:span>',
		'<h:span class="italic">\\1</h:span>',
		'<h:span style="color: \\1;">\\2</h:span>',
		'<h:a href="\\1">\\2</h:a>',
		'<h:br />',
		'&amp;',
	);
	$str = preg_replace($pattern, $replace_ns, $string);
	return $str;
}

function safeAmpersand($string) {
	return preg_replace('/&(?!amp;)/', '&amp;', $string);
}

$versions = array('0.9.1', '0.10.0', '0.11.0', '0.11.1', '0.12.0', 'SVN');
$timestamps = array(
	'0.9.1' => mktime(0, 0, 0, 09, 18, 2008),
	'0.10.0' => mktime(0, 0, 0, 09, 18, 2008),
	'0.11.0' => mktime(0, 0, 0, 09, 18, 2008),
	'0.11.1' => mktime(0, 0, 0, 09, 18, 2008),
	'0.12.0' => mktime(0, 0, 0, 09, 18, 2008),
);

foreach ($versions as $ver) {
	require_once ("../data/compatibility/compat-{$ver}.inc");

	print "converting: compat-{$ver}.inc ... ";

	$xml = "<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?>\n";
	$xml .= "<compatibility xmlns:h=\"http://www.w3.org/TR/xhtml1/\">\n";
	foreach ($gamesCompanies as $name => $games) {
		$xml .= "\t<company>\n";
		$name_safe = safeAmpersand($name);
		$xml .= "\t\t<name>{$name_safe}</name>\n";
		$xml .= "\t\t<games>\n";
		foreach ($games as $gname => $values) {
			$gname_safe = safeAmpersand($gname);
			$xml .= "\t\t\t<game>\n";
			$xml .= "\t\t\t\t<name>$gname_safe</name>\n";
			$xml .= "\t\t\t\t<target>{$values[0]}</target>\n";
			$xml .= "\t\t\t\t<percent>{$values[1]}</percent>\n";
			if (isset($notes[$values[0]])) {
				//$note = str_replace("<br>", "\n\t\t\t\t\t", $notes[$values[0]]);
				$note = html2xhtml($notes[$values[0]]);
				/* The notes for Loom are the same for all versions, add the fixed HTML straight away. */
				if ($target === 'Loom') {
					$note = "No known issues, game is completable.<h:br />- Amiga, Atari ST, FM-TOWNS, Macintosh and PC versions supported by this target<h:br />- The Roland update from LucasArts is required for MIDI support in the EGA version<h:br />- No music or sound effects in the Macintosh version<h:br />- There is no support for the Macintosh interface<h:br />- Fades are seemingly different in the FM-TOWNS version<h:br />- Text palette sometimes incorrect in the FM-TOWNS version<h:br />- Distaff occasionally pink in the FM-TOWNS version<h:br />- Use boot parameter to choose difficulty in the FM-TOWNS version:<h:ul><h:li>0 practice (default)</h:li><h:li>1 standard</h:li><h:li>2 expert</h:li></h:ul><h:br />- Kanji version requires the FM-TOWNS Font ROM";
				}
				$xml .= "\t\t\t\t<notes>\n";
				$xml .= "\t\t\t\t\t{$note}\n";
				$xml .= "\t\t\t\t</notes>\n";
			}
			$xml .= "\t\t\t</game>\n";
		}
		$xml .= "\t\t</games>\n";
		$xml .= "\t</company>\n";
	}
	$xml .= "</compatibility>\n";
	unset ($gamesCompanies, $notes);

	$filename = "../data/compatibility/compat-{$ver}.xml";
	file_put_contents($filename, $xml);
	if (array_key_exists($ver, $timestamps)) {
		touch($filename, $timestamps[$ver]);
	}

	print "done, written to compat-{$ver}.xml\n";
}
?>
