<?php
require_once('Models/BasicModel.php');
require_once('Objects/CompatGame.php');
require_once('XMLParser.php');
/**
 * The CompatibilityModel class will generate CompatGame objects.
 */
abstract class CompatibilityModel extends BasicModel {
	/* Get all the groups and the respectively demos for the specified ScummVM version. */
	static public function getAllData($version) {
		if (!is_string($version)) {
			return false;
		}
		$fname = DIR_COMPAT . "/compat-{$version}.xml";
		$parser = new XMLParser();
		if (!($a = $parser->parseByFilename($fname, true))) {
			return false;
		}
		$entries = array();
		foreach ($a as $key => $value) {
			$games = array();
			foreach ($value['games'] as $data) {
				$games[] = new CompatGame($data);
			}
			$entries[$value['name']] = $games;
		}
		return $entries;
	}

	/* Get version numbers for all available compatibility charts, excluding the SVN charts. */
	static public function getAllVersions() {
		$dir = DIR_COMPAT;
		if (!($files = scandir($dir))) {
			return false;
		}
		$files = Utils::getFromArrayByPatternMatch($files, '.*\.xml');
		$dates = array();
		foreach ($files as $file) {
			/* Always exclude the SVN-chart */
			if (strpos($file, 'SVN') === false) {
				$dates[] = substr($file, (strpos($file, '-') + 1), -4);
			}
		}
		natsort($dates);
		$dates = array_reverse($dates);
		return $dates;
	}

	/* Get a specific CompatGame-object for the requested version. */
	static public function getGameData($version, $target) {
		if (!is_string($version) || !is_string($target)) {
			return false;
		}
		if (!($all_games = self::getAllData($version))) {
			return false;
		}
		$g = null;
		foreach ($all_games as $company => $games) {
			foreach ($games as $game) {
				if ($game->getTarget() == $target) {
					$g = $game;
					break;
				}
			}
			if ($g != null) {
				break;
			}
		}
		return (is_null($g) ? false : $g);
	}
}
?>
