<?php
require_once('Models/BasicModel.php');
require_once('Objects/News.php');
/**
 * The NewsModel class will generate News objects
 */
abstract class NewsModel extends BasicModel {
	/* Get a list of all the available news dates. */
	static public function getListOfNewsDates() {
		if (!($files = scandir(DIR_NEWS))) {
			return false;
		}
		$files = Utils::getFromArrayByPatternMatch($files, '.*\.xml');
		$dates = array();
		foreach ($files as $file) {
			if (substr($file, -4) == '.xml') {
				$dates[] = substr($file, 0, -4);
			}
		}
		sort($dates, SORT_NUMERIC);
		return $dates;
	}

	/* Get all news items ordered by date, descending. */
	static public function getAllNews() {
		if (!($files = scandir(DIR_NEWS))) {
			return false;
		}
		$files = Utils::getFromArrayByPatternMatch($files, '.*\.xml');
		$news = array();
		foreach ($files as $file) {
			if (!($data = @file_get_contents(DIR_NEWS . "/{$file}"))) {
				continue;
			}
			$news[] = new News($data);
		}
		return array_reverse($news);
	}

	/* Get the latest number of news items, or if no number is specified get all news items. */
	static public function getLatestNews($num=-1) {
		if ($num == -1) {
			return NewsModel::getAllNews();
		} else {
			if (!($newslist = NewsModel::getListOfNewsDates())) {
				return false;
			}
			rsort($newslist, SORT_NUMERIC);
			$newslist = array_slice($newslist, 0, $num);
			$news = array();
			foreach ($newslist as $date) {
				$news[] = NewsModel::getByDate($date);
			}
			return $news;
		}
	}

	/* Get the news item that was posted on a specific date. */
	static public function getByDate($date) {
		if ($date == null || !is_numeric($date) || strlen($date) != 8) {
			return false;
		}
		if (!is_file(($fname = DIR_NEWS . "/{$date}.xml"))
			|| !is_readable($fname) || !($data = @file_get_contents($fname))) {
			return false;
		}
		$news = new News($data);
		return $news;
	}
}
?>
